@@ -18,25 +18,49 @@ along with OpenSesame.  If not, see <http://www.gnu.org/licenses/>.
 """
 
 from libopensesame.py3compat import *
+import os
+import sys
+import platform
 
 
-def opensesame():
+def set_paths():
 
-	import os, sys, platform
+	from qtpy import QtCore
+	from libopensesame import misc
 	# Add the folder that contains the OpenSesame modules to the path. This is
 	# generally only necessary if OpenSesame is directly run from source,
 	# instead from an installation.
 	if os.path.exists(os.path.join(os.getcwd(), 'libopensesame')):
 		sys.path.insert(0, os.getcwd())
+	# Add the Qt plugin folders to the library path, if they exists. Where
+	# these folders are depends on the version of Qt4, but these are two
+	# possible locations.
+	qt_plugin_path = os.path.join(
+		os.path.dirname(sys.executable), 'Library', 'plugins')
+	if os.path.isdir(qt_plugin_path):
+		QtCore.QCoreApplication.addLibraryPath(
+			safe_decode(qt_plugin_path, enc=misc.filesystem_encoding())
+		)
+	qt_plugin_path = os.path.join(
+		os.path.dirname(sys.executable), 'Library', 'lib', 'qt4', 'plugins')
+	if os.path.isdir(qt_plugin_path):
+		QtCore.QCoreApplication.addLibraryPath(
+			safe_decode(qt_plugin_path, enc=misc.filesystem_encoding())
+		)
+
+
+def opensesame():
+
+	set_paths()
 	# Support for multiprocessing when packaged
 	# In OS X the multiprocessing module is horribly broken, but a fixed
 	# version has been released as the 'billiard' module
 	if platform.system() == 'Darwin':
-		# Use normal multirpocessing module from python 3.4 and on
-		if sys.version_info >= (3,4):
+		# Use normal multiprocessing module from python 3.4 and on
+		if sys.version_info >= (3, 4):
 			from multiprocessing import freeze_support, set_start_method
 			freeze_support()
-			set_start_method('forkserver')
+			set_start_method('spawn')
 		else:
 			from billiard import freeze_support, forking_enable
 			freeze_support()
@@ -45,16 +69,13 @@ def opensesame():
 		from multiprocessing import freeze_support
 		freeze_support()
 	# Parse the (optional) environment file that contains special paths, etc.
-	from libopensesame.misc import resource, filesystem_encoding, \
-		parse_environment_file
+	from libopensesame.misc import parse_environment_file
 	parse_environment_file()
 	# Force the new-style Qt API
 	import sip
 	import qtpy
 	sip.setapi('QString', 2)
 	sip.setapi('QVariant', 2)
-	# Load debug package (this must be after the working directory change)
-	from libopensesame import debug
 	# Do the basic window initialization
 	from qtpy.QtWidgets import QApplication
 	# From Qt 5.6 on, QtWebEngine is the default way to render web pages
@@ -73,7 +94,7 @@ def opensesame():
 	opensesame.__script__ = __file__
 	app.processEvents()
 	# Install the translator. For some reason, the translator needs to be
-	# instantiated here and not in the set_locale() function, otherwise the	
+	# instantiated here and not in the set_locale() function, otherwise the
 	# locale is ignored.
 	from qtpy.QtCore import QTranslator
 	translator = QTranslator()
@@ -82,7 +103,6 @@ def opensesame():
 	# GUI initialization.
 	opensesame.resume_init()
 	opensesame.restore_window_state()
-	opensesame.refresh()
 	opensesame.show()
 	# Added for OS X, otherwise Window will not appear
 	opensesame.raise_()
@@ -92,14 +112,9 @@ def opensesame():
 
 def opensesamerun():
 
-	# First, load a minimum number of modules and show an empty app window. This
-	# gives the user the feeling of a snappy response.
-	import os, sys
-	# Add the folder that contains the OpenSesame modules to the path. This is
-	# generally only necessary if OpenSesame is directly run from source,
-	# instead from an installation.
-	if os.path.exists(os.path.join(os.getcwd(), 'libopensesame')):
-		sys.path.insert(0, os.getcwd())
+	set_paths()
+	from libopensesame.oslogging import oslogger
+	oslogger.start(u'gui')
 	import libopensesame.misc
 	libopensesame.misc.parse_environment_file()
 	import libopensesame.experiment
@@ -119,8 +134,10 @@ def opensesamerun():
 			sip.setapi('QVariant', 2)
 			from qtpy import QtGui, QtCore, QtWidgets
 		except:
-			libopensesame.misc.messagebox(u"OpenSesame Run",
-				u"Incorrect or missing options.\n\nRun 'opensesame --help' from a terminal (or command prompt) to see a list of available options, or install Python Qt4 to enable the graphical user interface.")
+			libopensesame.misc.messagebox(u"OpenSesame Run", u"Incorrect or "
+				u"missing options.\n\nRun 'opensesame --help' from a terminal "
+				u"(or command prompt) to see a list of available options, or "
+				u"install Python Qt4 to enable the graphical user interface.")
 			sys.exit()
 		# Create the GUI and show it
 		import libqtopensesame.qtopensesamerun
@@ -145,6 +162,10 @@ def opensesamerun():
 		logfile = safe_decode(logfile,
 			enc=libopensesame.misc.filesystem_encoding(), errors=u'ignore')
 
+	experiment_path = safe_decode(
+		os.path.abspath(options.experiment),
+		enc=libopensesame.misc.filesystem_encoding()
+	)
 	if options.debug:
 		# In debug mode, don't try to catch any exceptions
 		exp = libopensesame.experiment.experiment(u"Experiment",
@@ -156,8 +177,6 @@ def opensesamerun():
 		exp.end()
 	else:
 		# Try to parse the experiment from a file
-		experiment_path = safe_decode(os.path.abspath(options.experiment),
-			enc=libopensesame.misc.filesystem_encoding())
 		try:
 			exp = libopensesame.experiment.experiment(u"Experiment",
 				experiment, experiment_path=experiment_path)
