@@ -24,6 +24,7 @@ from libqtopensesame.misc.config import cfg
 from libqtopensesame.items import experiment
 from libopensesame import debug, metadata
 from libopensesame.exceptions import osexception
+from libopensesame.oslogging import oslogger
 import libopensesame.experiment
 import libopensesame.plugins
 import libopensesame.misc
@@ -32,6 +33,7 @@ import sys
 import warnings
 from libqtopensesame.misc.translate import translation_context
 _ = translation_context(u'qtopensesame', category=u'core')
+oslogger.start(u'gui')
 
 class qtopensesame(QtWidgets.QMainWindow, base_component):
 
@@ -67,7 +69,7 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		self.lock_refresh = False
 		self.unsaved_changes = False
 		self._run_status = u'inactive'
-		self.block_close_event = False		
+		self.block_close_event = False
 		self.parse_command_line()
 		self.restore_config()
 
@@ -85,17 +87,6 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		# icons by default.
 		QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_DontShowIconsInMenus,
 			False)
-		# Add the Qt plugin folders to the library path, if they exists. Where
-		# these folders are depends on the version of Qt4, but these are two
-		# possible locations.
-		qt_plugin_path = os.path.join(
-			os.path.dirname(sys.executable), 'Library', 'plugins')
-		if os.path.isdir(qt_plugin_path):
-			QtCore.QCoreApplication.addLibraryPath(qt_plugin_path)
-		qt_plugin_path = os.path.join(
-			os.path.dirname(sys.executable), 'Library', 'lib', 'qt4', 'plugins')
-		if os.path.isdir(qt_plugin_path):
-			QtCore.QCoreApplication.addLibraryPath(qt_plugin_path)
 		# Do a few things to customize QProgEdit behavior:
 		# - Register the bundled monospace font (Droid Sans Mono)
 		# - Make sure that QProgEdit doesn't complain about some standard names
@@ -111,11 +102,11 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		random.seed()
 
 		# Check the filesystem encoding for debugging purposes
-		debug.msg(u'filesystem encoding: %s' % misc.filesystem_encoding())
+		oslogger.debug(u'filesystem encoding: %s' % misc.filesystem_encoding())
 
 		# # Parse the command line
 		# self.parse_command_line()
-		# 
+		#
 		# # Restore the configuration
 		# self.restore_config()
 		self.set_style()
@@ -158,6 +149,7 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		self.ui.action_run_in_window.triggered.connect(
 			self.run_experiment_in_window)
 		self.ui.action_run_quick.triggered.connect(self.run_quick)
+		self.ui.action_kill.triggered.connect(self.kill_experiment)
 		self.ui.action_enable_auto_response.triggered.connect(
 			self.set_auto_response)
 		self.ui.action_close_current_tab.triggered.connect(
@@ -221,16 +213,14 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 				fd.read())
 		self.experiment.build_item_tree()
 		self.ui.itemtree.default_fold_state()
-
 		# Miscellaneous initialization
 		self.restore_state()
 		self.update_recent_files()
 		self.set_unsaved(False)
 		self.init_custom_fonts()
-
-		# Initialize extensions
 		self.extension_manager = extension_manager(self)
 		self.extension_manager.fire(u'startup')
+		self.ui.console.start()
 
 	def focus_debug_window(self):
 
@@ -271,17 +261,19 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		"""
 
 		from libqtopensesame.widgets.font_widget import font_widget
-		for font in font_widget.font_list + [
-				u'RobotoCondensed-Regular',
-				u'RobotoSlab-Regular',
-				u'RobotoMono-Regular',
-				u'Roboto-Regular']:
+		# The last element of font_list is the "other" entry
+		for font in font_widget.font_list[:-1] + [
+			u'RobotoCondensed-Regular',
+			u'RobotoSlab-Regular',
+			u'RobotoMono-Regular',
+			u'Roboto-Regular'
+		]:
 			try:
 				ttf = self.experiment.resource(u'%s.ttf' % font)
-			except:
-				debug.msg(u'failed to find %s' % font)
+			except FileNotFoundError:
+				oslogger.error(u'failed to find %s' % font)
 			else:
-				debug.msg(u'registering %s (%s)' % (font, ttf))
+				oslogger.debug(u'registering %s (%s)' % (font, ttf))
 				id = QtGui.QFontDatabase.addApplicationFont(ttf)
 				families = QtGui.QFontDatabase.applicationFontFamilies(id)
 				if families:
@@ -359,7 +351,7 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		cfg.parse_cmdline_args(self.options._config)
 		self.recent_files = []
 		if self.options.start_clean:
-			debug.msg(u'Not restoring state')
+			oslogger.info(u'Not restoring state')
 			self.theme.set_toolbar_size(cfg.toolbar_size)
 			return
 		self.resize(cfg.size)
@@ -378,10 +370,10 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		if hasattr(recent_files, u"split"):
 			for path in recent_files.split(u";;"):
 				if os.path.exists(path):
-					debug.msg(u"adding recent file '%s'" % path)
+					oslogger.debug(u"adding recent file '%s'" % path)
 					self.recent_files.append(path)
 				else:
-					debug.msg(u"missing recent file '%s'" % path)
+					oslogger.debug(u"missing recent file '%s'" % path)
 		self.ui.action_enable_auto_response.setChecked(
 			self.experiment.auto_response)
 		self.ui.action_onetabmode.setChecked(cfg.onetabmode)
@@ -402,7 +394,7 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		"""
 
 		if self.options.start_clean:
-			debug.msg(u'Not restoring window state')
+			oslogger.info(u'Not restoring window state')
 			return
 		self.restoreState(cfg._initial_window_state)
 		self.restoreGeometry(cfg._initial_window_geometry)
@@ -443,17 +435,16 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 
 		if cfg.style in QtWidgets.QStyleFactory.keys():
 			self.setStyle(QtWidgets.QStyleFactory.create(cfg.style))
-			debug.msg(u"using style '%s'" % cfg.style)
+			oslogger.debug(u"using style '%s'" % cfg.style)
 		else:
-			debug.msg(u"ignoring unknown style '%s'" % cfg.style)
 			cfg.style = u''
-			
+
 	def set_locale(self, translator):
-		
+
 		""""
 		desc:
 			Sets the application language.
-		
+
 		arguments:
 			translator:
 				desc:	For some reason, the QTranslator object needs to be
@@ -700,7 +691,7 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		keywords:
 			dummy:		Dummy argument passed by event handler.
 			path:
-			 	desc:	The path to the file. If None, a file dialog is
+				desc:	The path to the file. If None, a file dialog is
 						presented.
 				type:	[str, NoneType]
 			add_to_recent:
@@ -709,34 +700,42 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 				type:	bool
 		"""
 
+		from libopensesame.osexpfile import osexpreader
+
 		if not self.save_unsaved_changes():
 			self.ui.tabwidget.open_general()
 			return
 		if path is None:
-			path = QtWidgets.QFileDialog.getOpenFileName(self.ui.centralwidget,
-				_(u"Open file"), filter=self.open_file_filter,
-				directory=cfg.file_dialog_path)
+			path = QtWidgets.QFileDialog.getOpenFileName(
+				self.ui.centralwidget,
+				_(u"Open file"),
+				filter=self.open_file_filter,
+				directory=cfg.file_dialog_path
+			)
 		# In PyQt5, the QFileDialog.getOpenFileName returns a tuple instead of
 		# a string, of which the first position contains the path. check for that
 		# here.
-		if isinstance(path,tuple):
+		if isinstance(path, tuple):
 			path = path[0]
-		if path is None or path == u'' or ( \
-			not path.lower().endswith(u'.opensesame') and \
-			not  path.lower().endswith(u'.opensesame.tar.gz') and \
-			not path.lower().endswith(u'.osexp')):
+		if path is None or not osexpreader.valid_extension(path):
 			return
 		self.set_busy()
 		self.ui.tabwidget.close_all(avoid_empty=False)
 		cfg.file_dialog_path = os.path.dirname(path)
 		try:
-			exp = experiment.experiment(self, u"Experiment", path,
-				experiment_path=os.path.dirname(path))
+			exp = experiment.experiment(
+				self,
+				u"Experiment",
+				path,
+				experiment_path=os.path.dirname(path)
+			)
 		except Exception as e:
 			if not isinstance(e, osexception):
 				e = osexception(msg=u'Failed to open file', exception=e)
-			md = _(u'# Failed to open\n\nFailed to open the file for the '
-				u'following reason:\n\n- ') + e.markdown()
+			md = _(
+				u'# Failed to open\n\nFailed to open the file for the '
+				u'following reason:\n\n- '
+			) + e.markdown()
 			self.tabwidget.open_markdown(md)
 			self.ui.console.write(e)
 			self.set_busy(False)
@@ -764,9 +763,13 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 		# Process non-fatal errors
 		if exp.items.error_log:
 			self.tabwidget.open_markdown(
-				_(u'Errors occurred while opening the file:\n\n') + \
-				u'\n---\n'.join([exc.markdown() for exc in exp.items.error_log]),
-				title=_(u'Error'), icon=u'dialog-error')
+				_(u'Errors occurred while opening the file:\n\n') +
+				u'\n---\n'.join(
+					[exc.markdown() for exc in exp.items.error_log]
+				),
+				title=_(u'Error'),
+				icon=u'dialog-error'
+			)
 			self.window_message(u"New experiment")
 			self.current_path = None
 
@@ -819,7 +822,10 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 			self.set_busy(False)
 		except Exception as e:
 			self.ui.console.write(e)
-			self.experiment.notify(_(u"Failed to save file. Error: %s") % e)
+			self.experiment.notify(
+				_(u"Failed to save file. Error: %s")
+				% safe_decode(e)
+			)
 			self.set_busy(False)
 			return
 		self.update_recent_files()
@@ -900,7 +906,7 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 			height:		The display height in pixels.
 		"""
 
-		debug.msg(u"changing resolution to %d x %d" % (width, height))
+		oslogger.debug(u"changing resolution to %d x %d" % (width, height))
 		try:
 			script = self.experiment.to_string()
 		except Exception as e:
@@ -947,10 +953,19 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 				if item not in done:
 					done.append(item)
 					if self.experiment.items[item].get_ready():
-						debug.msg(u"'%s' did something" % item)
+						oslogger.debug(u"'%s' did something" % item)
 						redo = True
 						break
 
+	def kill_experiment(self):
+
+		"""Tries to kill a running experiment. This is not supported by all
+		runners.
+		"""
+
+		self._runner.kill()
+		self.ui.action_kill.setDisabled(True)
+
 	def run_experiment(self, dummy=None, fullscreen=True, quick=False):
 
 		"""
@@ -966,11 +981,12 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 						testing the experiment. (default=False)
 		"""
 
+		self.get_ready()
 		self.enable(False)
 		print(u'\n')
-		debug.msg(u'using %s runner' % cfg.runner)
-		_runner = self.runner_cls(self)
-		_runner.run(quick=quick, fullscreen=fullscreen,
+		oslogger.debug(u'using %s runner' % cfg.runner)
+		self._runner = self.runner_cls(self)
+		self._runner.run(quick=quick, fullscreen=fullscreen,
 			auto_response=self.experiment.auto_response)
 		self.enable(True)
 
@@ -1019,12 +1035,17 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 				type:	bool
 		"""
 
-		if sys.platform == 'darwin':
-			return
 		self.block_close_event = not enabled
 		self.ui.dock_overview.setEnabled(enabled)
 		self.ui.centralwidget.setEnabled(enabled)
-		self.ui.toolbar_main.setEnabled(enabled)
+		for action in self.ui.toolbar_main.actions():
+			# The kill action should be enabled when the experiment is running
+			# and the runner supports killing
+			action.setEnabled(
+				not enabled and self.runner_cls.supports_kill
+				if action.objectName() == u'action_kill'
+				else enabled
+			)
 		self.ui.toolbar_items.setEnabled(enabled)
 		self.ui.menubar.setEnabled(enabled)
 		self.ui.dock_pool.setEnabled(enabled)
@@ -1038,7 +1059,7 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 			but has been deprecated.
 		"""
 
-		debug.msg(reason=u'deprecated')
+		oslogger.warning(u'qtopensesame.refresh() is deprecated')
 
 	def _id(self):
 
@@ -1055,21 +1076,21 @@ class qtopensesame(QtWidgets.QMainWindow, base_component):
 
 	@property
 	def read_only(self):
-		
+
 		"""
 		desc:
 			Getter property for toggling the save action when setting.
 		"""
-						
+
 		return self._read_only
-		
+
 	@read_only.setter
 	def read_only(self, read_only):
-		
+
 		"""
 		desc:
 			Setter property for toggling the save action.
-		"""		
-		
+		"""
+
 		self._read_only = read_only
 		self.ui.action_save.setEnabled(not read_only)
