@@ -18,7 +18,6 @@ along with OpenSesame.  If not, see <http://www.gnu.org/licenses/>.
 """
 
 from libopensesame.py3compat import *
-import os
 import sys
 import time
 from libqtopensesame.runners import base_runner
@@ -29,6 +28,8 @@ class multiprocess_runner(base_runner):
 
 	"""Runs an experiment in another process using multiprocessing."""
 
+	supports_kill = True
+
 	def execute(self):
 
 		"""See base_runner.execute()."""
@@ -44,7 +45,6 @@ class multiprocess_runner(base_runner):
 			import multiprocessing
 
 		from libqtopensesame.misc import process, _
-		from libopensesame import misc
 
 		self._workspace_globals = {}
 		self.channel = multiprocessing.Queue()
@@ -54,10 +54,12 @@ class multiprocess_runner(base_runner):
 		except Exception as e:
 			return osexception(_(u'Failed to initialize experiment process'),
 				exception=e)
+		self.console.set_workspace_globals({u'process' : self.exp_process})
 		# Start process!
 		self.exp_process.start()
 		# Wait for experiment to finish.
 		# Listen for incoming messages in the meantime.
+		finished = False
 		while self.exp_process.is_alive() or not self.channel.empty():
 			# We need to process the GUI. To make the GUI feel more responsive
 			# during pauses, we refresh the GUI more often when paused.
@@ -87,6 +89,8 @@ class multiprocess_runner(base_runner):
 			# indicates whether the experiment should be paused or resumed.
 			if isinstance(msg, dict):
 				self._workspace_globals = msg
+				if u'__kill__' in msg:
+					self.exp_process.kill()
 				if u'__heartbeat__' in msg:
 					self.console.set_workspace_globals(msg)
 					self.main_window.extension_manager.fire(u'heartbeat')
@@ -95,14 +99,27 @@ class multiprocess_runner(base_runner):
 						self.pause()
 					else:
 						self.resume()
+				elif u'__finished__' in msg:
+					finished = True
 				continue
 			# Anything that is not a string, not an Exception, and not None is
 			# unexpected
 			return osexception(
 				u"Illegal message type received from child process: %s (%s)" \
 				% (msg, type(msg)))
-		# Return None if experiment finished without problems
-		return None
+		if not finished:
+			if self.exp_process.killed:
+				return osexception(u'The experiment process was killed.')
+			else:
+				return osexception(u'Python seems to have crashed. This should not '
+					u'happen. If Python crashes often, please report it on the '
+					u'OpenSesame forum.')
+
+	def kill(self):
+
+		"""See base_runner."""
+
+		self.exp_process.kill()
 
 	def workspace_globals(self):
 
